#include "gpx2shp.h"
#include <assert.h>

void
writeRteAttribute(g2sprop * pr, trkType * trt, int iShape)
{
#ifdef DEBUG
    printf("(writeRteAttribute)\n");
#endif
    int             acn = 8;	/* addtional column number like length,
				 * time and speed */
    if (!DBFWriteIntegerAttribute(pr->hDBF_rte, iShape, 0, iShape))
	failToWriteAttr(iShape, 0, "writeRteAttribute");
    if (!DBFWriteStringAttribute(pr->hDBF_rte, iShape, 1, trt->name))
	failToWriteAttr(iShape, 1, "writeRteAttribute");
    if (!DBFWriteStringAttribute(pr->hDBF_rte, iShape, 2, trt->cmt))
	failToWriteAttr(iShape, 2, "writeRteAttribute");
    if (!DBFWriteStringAttribute(pr->hDBF_rte, iShape, 3, trt->desc))
	failToWriteAttr(iShape, 3, "writeRteAttribute");
    if (!DBFWriteStringAttribute(pr->hDBF_rte, iShape, 4, trt->src))
	failToWriteAttr(iShape, 4, "writeRteAttribute");
    if (!DBFWriteStringAttribute(pr->hDBF_rte, iShape, 5, trt->link))
	failToWriteAttr(iShape, 5, "writeRteAttribute");
    if (!DBFWriteIntegerAttribute(pr->hDBF_rte, iShape, 6, trt->number))
	failToWriteAttr(iShape, 6, "writeRteAttribute");
    if (!DBFWriteStringAttribute(pr->hDBF_rte, iShape, 7, trt->type))
	failToWriteAttr(iShape, 7, "writeRteAttribute");
    if (pr->needsLength) {
	if (!DBFWriteDoubleAttribute
	    (pr->hDBF_rte, iShape, acn, trt->length))
	    failToWriteAttr(iShape, acn, "writeRteAttribute");
	acn++;
    }
    if (pr->needsTime) {
	if (!DBFWriteDoubleAttribute(pr->hDBF_rte, iShape, acn, trt->time))
	    failToWriteAttr(iShape, acn, "writeRteAttribute");
	acn++;
    }
    if (pr->needsSpeed) {
	if (!DBFWriteDoubleAttribute
	    (pr->hDBF_rte, iShape, acn, trt->speed))
	    failToWriteAttr(iShape, acn, "writeRteAttribute");
    }
}

void
addRteField(g2sprop * pr)
{
#ifdef DEBUG
    printf("(addRteField)\n");
#endif
    char            columnName[16];
    DBFAddField(pr->hDBF_rte, "shapeID", FTInteger, 8, 0);
    DBFAddField(pr->hDBF_rte, "name", FTString, NAMELENGTH, 0);
    DBFAddField(pr->hDBF_rte, "cmt", FTString, COMMENTLENGTH, 0);
    DBFAddField(pr->hDBF_rte, "desc", FTString, COMMENTLENGTH, 0);
    DBFAddField(pr->hDBF_rte, "src", FTString, COMMENTLENGTH, 0);
    DBFAddField(pr->hDBF_rte, "link", FTString, FILENAMELENGTH, 0);
    DBFAddField(pr->hDBF_rte, "number", FTInteger, 8, 0);
    DBFAddField(pr->hDBF_rte, "type", FTString, TYPELENGTH, 0);
    if (pr->needsLength) {
	sprintf(columnName, "l(%s)", pr->lengthUnit);
	DBFAddField(pr->hDBF_rte, columnName, FTDouble, 17, 4);
    }
    if (pr->needsTime) {
	sprintf(columnName, "t(%s)", "sec");
	DBFAddField(pr->hDBF_rte, columnName, FTDouble, 12, 2);
    }
    if (pr->needsSpeed) {
	sprintf(columnName, "s(%s/%s)", pr->speedLengthUnit,
		pr->speedTimeUnit);
	DBFAddField(pr->hDBF_rte, columnName, FTDouble, 12, 2);
    }
}

void
getRteAsPoints(g2sprop * pr, xmlNodePtr cur)
{
#ifdef DEBUG
    printf("(wptroot)\n");
#endif
    static int      isFirst = 0;
    char            output_rte_pnt[FILENAMELENGTH];

    if (!isFirst) {
	strcpy(output_rte_pnt, pr->output);
	strcat(output_rte_pnt, "_rte_pnt");

	if (pr->is3d) {
	    pr->hSHP_rte_pnt = SHPCreate(output_rte_pnt, SHPT_POINTZ);
	} else {
	    pr->hSHP_rte_pnt = SHPCreate(output_rte_pnt, SHPT_POINT);
	}
	pr->hDBF_rte_pnt = DBFCreate(output_rte_pnt);
	addWptField(pr->hDBF_rte_pnt);
	isFirst = 1;
    }
    getWpt(pr->hSHP_rte_pnt, pr->hDBF_rte_pnt, pr, cur);
    return;
}

void
getRtept(g2sprop * pr, xmlNodePtr cur, trkType * trt)
{
    xmlChar        *lat;
    xmlChar        *lon;
    static int      iShape;
    int             elements,
                    i = 0,
                    isFirst = 1;
    char            t[TIMELENGTH],
                    _t[TIMELENGTH];
    char            ts[TIMELENGTH],
                    te[TIMELENGTH];
    xmlNodePtr      _cur;
    SHPObject      *shpobj;
    wptType         wpt;
    double         *x,
                   *y,
                   *z;

#ifdef DEBUG
    printf("(getRtept)\n");
#endif
    if (pr->verbose) {
	printf("      <rtept>\n");
    }
    initWptType(&wpt);
    elements = getElementCount(cur);
    x = malloc(3 * elements * sizeof(*x));
    assert(x);
    y = &x[elements];
    z = &x[2 * elements];
    while (cur != NULL) {
	if ((!xmlStrcmp(cur->name, (const xmlChar *) "rtept"))) {
	    _cur = cur;
	    lat = xmlGetProp(cur, "lat");
	    lon = xmlGetProp(cur, "lon");
	    x[i] = atof(lon);
	    y[i] = atof(lat);
	    getWptChildren(pr, _cur, &wpt);
	    z[i] = wpt.ele;
	    strcpy(t, wpt.time);
	    if (isFirst) {
		strcpy(ts, t);
		strcpy(_t, t);
		isFirst = 0;
	    } else {
		if (pr->isSeparated)
		    separatedPath(x[i - 1], y[i - 1], z[i - 1], _t, x[i],
				  y[i], z[i], t, iShape, pr, "rte");
		if (pr->isPoint)
		    getRteAsPoints(pr, cur);

		trt->length =
		    trt->length + getDistance(x[i - 1], y[i - 1], x[i],
					      y[i]);
		strcpy(_t, t);
	    }
	    i++;
	}
	cur = cur->next;
    }
    strcpy(te, t);
    trt->time = getTimeInterval(ts, te);
    trt->speed =
	getSpeed(trt->length, trt->time, pr->speed_to_meter,
		 pr->speed_to_sec);
    trt->length /= pr->length_to_meter;
    if (pr->verbose) {
	printf("        %i points\n", i);
	printf("        %f %s\n", trt->length, pr->lengthUnit);
	printf("        %f sec\n", trt->time);
	printf("        %f %s/%s\n", trt->speed, pr->speedLengthUnit,
	       pr->speedTimeUnit);
    }
    if (trt->length != 0) {
	if (pr->is3d) {
	    shpobj =
		SHPCreateObject(SHPT_ARCZ, iShape, 0, 0, 0, i, x, y, z,
				NULL);
	} else {
	    shpobj =
		SHPCreateObject(SHPT_ARC, iShape, 0, 0, 0, i, x, y, NULL,
				NULL);
	}
	iShape = SHPWriteObject(pr->hSHP_rte, -1, shpobj);
	writeRteAttribute(pr, trt, iShape);
	SHPDestroyObject(shpobj);
	pr->stats->rtelength += trt->length;
	pr->stats->rtepoints += i;
	pr->stats->rtecount++;
    } else {
	printf
	    ("This Track was not converted because no move recorded. %s:%s:%s:%s:%i\n",
	     trt->name, trt->cmt, trt->desc, trt->src, trt->number);
    }
/*
 * printf("%p,%p,%i,%f\n", pr, trt, iShape, length); 
 */
    free(x);
    return;
}

trkType
createRtetype(void)
{
#ifdef DEBUG
    printf("(createRtetype)\n");
#endif
    trkType         trt;
    trt.name[0] = '\0';
    trt.cmt[0] = '\0';
    trt.desc[0] = '\0';
    trt.src[0] = '\0';
    trt.link[0] = '\0';
    trt.number = 0;
    trt.type[0] = '\0';
    trt.length = 0;
    trt.speed = 0;
    return trt;
}

void
getRteAttr(g2sprop * pr, xmlNodePtr cur)
{
#ifdef DEBUG
    printf("(getRteseg)\n");
#endif
    trkType         trt = createRtetype();
    xmlChar        *element;
    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
    /*
     * Problem Tag <rtept>s should be the last in <rte>. 
     */
	if (xmlStrcmp(cur->name, (const xmlChar *) "rtept")) {
	    element =
		xmlNodeListGetString(pr->doc, cur->xmlChildrenNode, 1);
	    if ((!xmlStrcmp(cur->name, (const xmlChar *) "name")))
		strcpy(trt.name, element);
	    if ((!xmlStrcmp(cur->name, (const xmlChar *) "cmt")))
		strcpy(trt.cmt, element);
	    if ((!xmlStrcmp(cur->name, (const xmlChar *) "desc")))
		strcpy(trt.desc, element);
	    if ((!xmlStrcmp(cur->name, (const xmlChar *) "src")))
		strcpy(trt.src, element);
	    if ((!xmlStrcmp(cur->name, (const xmlChar *) "link")))
		strcpy(trt.link, element);
	    if ((!xmlStrcmp(cur->name, (const xmlChar *) "number")))
		trt.number = atoi(element);
	    if ((!xmlStrcmp(cur->name, (const xmlChar *) "type")))
		strcpy(trt.type, element);
	} else {
	    if (pr->verbose) {
		printf(" %s:%s:%s:%s:%i\n", trt.name, trt.cmt, trt.desc,
		       trt.src, trt.number);
		printf("    <rtept>\n");
	    }
	    getRtept(pr, cur, &trt);
	    return;
	}
	cur = cur->next;
    }
    return;
}

void
rteRoot(g2sprop * pr, xmlNodePtr cur)
{
    static int      isFirstRte = 0;
    char            output_rte[FILENAMELENGTH + 4];
#ifdef DEBUG
    printf("(rteRoot)\n");
#endif
    if (pr->verbose) {
	printf("  <rte>");
    }
    if (!isFirstRte) {
	strcpy(output_rte, pr->output);
	strcat(output_rte, "_rte");
    /*
     * printf ("%s, %s\n", pr->output, output_rte); 
     */

	if (pr->is3d)
	    pr->hSHP_rte = SHPCreate(output_rte, SHPT_ARCZ);
	else
	    pr->hSHP_rte = SHPCreate(output_rte, SHPT_ARC);

	pr->hDBF_rte = DBFCreate(output_rte);

	addRteField(pr);
	isFirstRte = 1;
    }
    getRteAttr(pr, cur);
    return;
}
