#include "gpx2shp.h"

void
writeSptAttribute(g2sprop * pr, DBFHandle hDBF, int iShape, int pShape,
		  double length, double ti, double speed)
{
    int             acn = 2;	/* addtional column number like length,
				 * time and speed */
    if (!DBFWriteIntegerAttribute(hDBF, iShape, 0, iShape))
	printf("%s shapeid: %i in %i\n",
	       "fail to write a iShape attribute.", iShape, __LINE__);
    if (!DBFWriteIntegerAttribute(hDBF, iShape, 1, pShape))
	printf("%s shapeid: %i in %i\n",
	       "fail to write a iShape attribute.", iShape, __LINE__);
    if (pr->needsLength) {
	if (!DBFWriteDoubleAttribute(hDBF, iShape, acn, length))
	    printf("%s shapeid: %i in %i\n",
		   "fail to write a length attribute.", iShape, __LINE__);
	acn++;
    }
    if (pr->needsTime) {
	if (!DBFWriteDoubleAttribute(hDBF, iShape, acn, ti))
	    printf("%s shapeid: %i in %i\n",
		   "fail to write a time attribute.", iShape, __LINE__);
	acn++;
    }
    if (pr->needsSpeed) {
	if (!DBFWriteDoubleAttribute(hDBF, iShape, acn, speed))
	    printf("%s shapeid: %i in %i\n",
		   "fail to write a speed attribute.", iShape, __LINE__);
    }
}

void
addSptField(g2sprop * pr, DBFHandle hDBF)
{
    char            columnName[16];
    DBFAddField(hDBF, "shapeid", FTInteger, 8, 0);
    DBFAddField(hDBF, "parentid", FTInteger, 8, 0);
    if (pr->needsLength) {
	sprintf(columnName, "l(%s)", pr->lengthUnit);
	DBFAddField(hDBF, columnName, FTDouble, 16, 4);
    }
    if (pr->needsLength) {
	sprintf(columnName, "t(%s)", pr->speedTimeUnit);
	DBFAddField(hDBF, columnName, FTDouble, 10, 2);
    }
    if (pr->needsLength) {
	sprintf(columnName, "s(%s/%s)", pr->speedLengthUnit,
		pr->speedTimeUnit);
	DBFAddField(hDBF, columnName, FTDouble, 8, 2);
    }
}

void
separatedPath(double _x, double _y, double _z, char *_t, double x,
	      double y, double z, char *t, int pShape, g2sprop * pr,
	      const char *type)
{
    double          length,
                    ti,
                    speed,
                    xx[2],
                    yy[2],
                    zz[2];
    SHPObject      *shpobj;
    static int      isTrkFirst = 1;
    static int      isRteFirst = 1;
    int             iShape = 0;
    char            output_spt[FILENAMELENGTH];
    SHPHandle       hSHP;
    DBFHandle       hDBF;
    xx[0] = _x;
    yy[0] = _y;
    zz[0] = _z;
    xx[1] = x;
    yy[1] = y;
    zz[1] = z;
    length = getDistance(_x, _y, x, y);
    ti = getTimeInterval(_t, t);
    speed = getSpeed(length, ti, pr->speed_to_meter, pr->speed_to_sec);
    length /= pr->length_to_meter;
    strcpy(output_spt, pr->output);
    strcat(output_spt, "_");
    strcat(output_spt, type);
    strcat(output_spt, "_spt");
    if (!strcmp(type, "trk")) {
	if (isTrkFirst) {
	    if (pr->is3d)
		pr->hSHP_trk_spt = SHPCreate(output_spt, SHPT_ARCZ);
	    else
		pr->hSHP_trk_spt = SHPCreate(output_spt, SHPT_ARC);
	    pr->hDBF_trk_spt = DBFCreate(output_spt);
	    addSptField(pr, pr->hDBF_trk_spt);
	    isTrkFirst = 0;
	}
	hSHP = pr->hSHP_trk_spt;
	hDBF = pr->hDBF_trk_spt;
    } else {
	if (isRteFirst) {
	    if (pr->is3d)
		pr->hSHP_rte_spt = SHPCreate(output_spt, SHPT_ARCZ);
	    else
		pr->hSHP_rte_spt = SHPCreate(output_spt, SHPT_ARC);
	    pr->hDBF_rte_spt = DBFCreate(output_spt);
	    addSptField(pr, pr->hDBF_rte_spt);
	    isRteFirst = 0;
	}
	hSHP = pr->hSHP_rte_spt;
	hDBF = pr->hDBF_rte_spt;
    }
    if (pr->is3d) {
	shpobj =
	    SHPCreateObject(SHPT_ARCZ, iShape, 0, 0, 0, 2, xx, yy, zz,
			    NULL);
    } else {
	shpobj =
	    SHPCreateObject(SHPT_ARC, iShape, 0, 0, 0, 2, xx, yy, NULL,
			    NULL);
    }
    iShape = SHPWriteObject(hSHP, -1, shpobj);
    SHPDestroyObject(shpobj);
    writeSptAttribute(pr, hDBF, iShape, pShape, length, ti, speed);
    return;
}
