#ifndef GPX2SHP_H_INCLUDED
#define GPX2SHP_H_INCLUDED
/* #define DEBUG */

#define FILENAMELENGTH 255 /* 255 is max length for dbf string column */
#define COMMENTLENGTH 255  /* 255 is max length for dbf string column */
#define NAMELENGTH 64
#define TIMELENGTH 32
#define TYPELENGTH 16 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <libgen.h>
#include <getopt.h>
#include <time.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include "shapefil.h"

/* waypoint structure from gpx-1.1 */
typedef struct wptType
{
  double ele;
  char time[TIMELENGTH];
  double magvar;
  double geoidheight;
  char name[NAMELENGTH];
  char cmt[COMMENTLENGTH];
  char desc[COMMENTLENGTH];
  char src[COMMENTLENGTH];
  char link[FILENAMELENGTH];
  char sym[TYPELENGTH];
  char type[TYPELENGTH];
  int sat;
  double hdop;
  double vdop;
  double ageofdgpsdata;
  double gdpsid;
  /* char extensions[32]; */
}
wptType;

/* trackpoint structure from gpx-1.1 */
typedef struct trkType
{
  char name[NAMELENGTH];
  char cmt[COMMENTLENGTH];
  char desc[COMMENTLENGTH];
  char src[COMMENTLENGTH];
  char link[FILENAMELENGTH];
  int number;
  char type[TYPELENGTH];
  /* char *extensions[32]; */
  double length;
  double time;
  double speed;
}
trkType;

/* route structure from gpx-1.1 */

/* source file status structure */
typedef struct g2sstats
{
  int trkpoints;
  int trkcount;
  double trklength;
  int rtepoints;
  int rtecount;
  double rtelength;
  int wptpoints;
}
g2sstats;

/* propaties structure for gpx2shp */
typedef struct g2sprop
{
  xmlDocPtr doc;
  DBFHandle hDBF_trk;
  SHPHandle hSHP_trk;
  DBFHandle hDBF_wpt;
  SHPHandle hSHP_wpt;
  DBFHandle hDBF_rte;
  SHPHandle hSHP_rte;
  DBFHandle hDBF_trk_spt;
  SHPHandle hSHP_trk_spt;
  DBFHandle hDBF_trk_pnt;
  SHPHandle hSHP_trk_pnt;
  DBFHandle hDBF_rte_spt;
  SHPHandle hSHP_rte_spt;
  DBFHandle hDBF_rte_pnt;
  SHPHandle hSHP_rte_pnt;
  int parseWpt;
  int parseTrk;
  int parseRte;
  int is3d;
  int isSeparated;
  int isPoint;
  int needsSpeed;
  int needsLength;
  int needsTime;
  int needsStats;
  int verbose;
  char sourcefile[FILENAMELENGTH];
  char output[FILENAMELENGTH];
  char ellipsoid[16];
  char lengthUnit[8];
  double length_to_meter;
  char speedLengthUnit[8];
  double speed_to_meter;
  char speedTimeUnit[8];
  int speed_to_sec;
  g2sstats *stats;
}
g2sprop;

/* getroot.c */
void parseDoc (g2sprop * pr);

/* getwpt.c */
void wptRoot (g2sprop * pr, xmlNodePtr cur);
void getWpt (SHPHandle hSHP, DBFHandle hDBF, g2sprop * pr, xmlNodePtr cur);
void getWptChildren (g2sprop * pr, xmlNodePtr cur, wptType *wpt);
void addWptField (DBFHandle hDBF);
void initWptType(wptType *wpt);

/* gettrk.c */
void trkRoot (g2sprop * pr, xmlNodePtr cur);

/* getrte.c */
void rteRoot (g2sprop * pr, xmlNodePtr cur);

/* getsep.c */
void
separatedPath (double _x, double _y, double _z, char *_t, double x, double y,
	       double z, char *t, int pShape, g2sprop * pr, const char *type);

/* utils.c */
void checkEllpsUnit (char *unit);
double checkLengthUnit (char *unit);
int checkTimeUnit (char *unit);
int getElementCount (xmlNodePtr cur);
double getTimeInterval (char *_t, char *t);
double getSpeed (double length, double ti, double to_meter, int to_sec);
double getDistance (double _x, double _y, double x, double y);
void closeFiles ();
void failToWriteAttr (int iShape, int col, char *source);

#endif
