#include "gpx2shp.h"

void
usage(char **argv)
{
/*
 * Shows a usage message
 */
    printf("Usage: %s gpxfile [options] [-o output basename]\n", argv[0]);
    fputs
	("-o, --output            Sets output basename. The default is (source file\n",
	 stdout);
    fputs("                        name) - (extention name)\n", stdout);
    fputs
	("-w, --waypoints         Converts only waypoints data from a gpx file.\n",
	 stdout);
    fputs
	("-t, --trackpoints       Converts only trackpoints data from a gpx file.\n",
	 stdout);
    fputs
	("-r, --routes            Converts only routes data from a gpx file.\n",
	 stdout);
    fputs
	("-a, --all               Converts all types of data from a gpx file.(default)\n",
	 stdout);
    fputs
	("-e, --as-edge           Makes a separated output by each edges.\n",
	 stdout);
    fputs
	("-p, --as-point          Makes a separated output by each points.\n",
	 stdout);
    fputs
	("-s, --stats             Shows simple statistics of the outputs.\n",
	 stdout);
    fputs
	("-L, --no-length         Removes length column from a waypoint or trackpoint\n",
	 stdout);
    fputs("                        attribute table.\n", stdout);
    fputs
	("-S, --no-speed          Removes speed column from a waypoint or trackpoint\n",
	 stdout);
    fputs("                        attribute table.\n", stdout);
    fputs
	("-T, --no-time           Removes time column from an attribute table.\n",
	 stdout);
    fputs
	("-3, --3d                Converts data using 3d format. (It's not compatible\n",
	 stdout);
    fputs("                        for Arcview 3.x.)\n", stdout);
    fputs
	("    --length-unit       Sets length unit from m,km,feet,mi and etc. default is m.\n",
	 stdout);
    fputs
	("                        You can see the unit list from \"geod -lu\" command.\n",
	 stdout);
    fputs
	("    --length-ellipsoid  Sets length ellipsoid like UGS84, clrk66. The default is\n",
	 stdout);
    fputs("                        UGS84.\n", stdout);
    fputs
	("                        You can see the unit list from \"geod -le\" command.\n",
	 stdout);
    fputs
	("    --speed-length-unit Sets length unit for speed. The default is km.\n",
	 stdout);
    fputs
	("                        You can see the unit list from \"geod -lu\" command.\n",
	 stdout);
    fputs
	("    --speed-time-unit   Sets time for speed unit. The default is hour.\n",
	 stdout);
    fputs
	("                        You can set from day, hour, min and sec.\n",
	 stdout);
    fputs("-v,  --verbose          Gives many messages.\n", stdout);
    fputs("-h,  --help             Shows this list.\n", stdout);
}

void
setDefault(g2sprop * pr)
{
/*
 * Sets default values to the properties 
 */
#ifdef DEBUG
    printf("(setDefault)\n");
#endif
    char           *pargv[2];
    char            ellps[16];
    char            _docname[FILENAMELENGTH];
    int             pargc = 2;

    if (!(pr->parseWpt | pr->parseTrk | pr->parseRte)) {
	pr->parseWpt = 1;
	pr->parseTrk = 1;
	pr->parseRte = 1;
    }
    if (pr->output[0] == '\0') {
	sscanf(pr->sourcefile, "%[^.]]", _docname);
	strcpy(pr->output, _docname);
    }
    if (pr->verbose)
	printf("output file base name: %s\n", pr->output);
    if (pr->ellipsoid[0] == '\0')
	strcpy(pr->ellipsoid, "WGS84");
    if (pr->lengthUnit[0] == '\0')
	strcpy(pr->lengthUnit, "m");
    if (pr->speedLengthUnit[0] == '\0')
	strcpy(pr->speedLengthUnit, "km");
    if (pr->speedTimeUnit[0] == '\0')
	strcpy(pr->speedTimeUnit, "hour");
    ellps[0] = '\0';
    strcat(ellps, "+ellps=");
    strcat(ellps, pr->ellipsoid);
    pargv[0] = ellps;
    pargv[1] = pr->lengthUnit;
    checkEllpsUnit(pr->ellipsoid);
    pr->speed_to_meter = checkLengthUnit(pr->speedLengthUnit);
    pr->length_to_meter = checkLengthUnit(pr->lengthUnit);
    pr->speed_to_sec = checkTimeUnit(pr->speedTimeUnit);
    geod_set(pargc, &pargv);
}

void
setOptions(int argc, char **argv, g2sprop * pr)
{
/*
 * Set options from command arguments
 */
    int             result;
#ifdef DEBUG
    printf("(setOptions)\n");
#endif
/*
 * option struct for getopt_long 
 */
    struct option const long_options[] = {
	{"waypoints", no_argument, 0, 'w'},
	{"trackpoints", no_argument, 0, 't'},
	{"routes", no_argument, 0, 'r'},
	{"output", required_argument, 0, 'o'},
	{"as-edge", no_argument, 0, 'e'},
	{"as-point", no_argument, 0, 'p'},
	{"stats", no_argument, 0, 's'},
	{"length-unit", required_argument, 0, '0'},
	{"length-ellipsoid", required_argument, 0, '4'},
	{"speed-length-unit", required_argument, 0, '1'},
	{"speed-time-unit", required_argument, 0, '2'},
	{"no-speed", no_argument, 0, 'S'},
	{"no-length", no_argument, 0, 'L'},
	{"no-time", no_argument, 0, 'T'},
	{"verbose", no_argument, 0, 'v'},
	{"all", no_argument, 0, 'a'},
	{"help", no_argument, 0, '?'},
    };
    if (argc <= 1) {
	fputs("There is no argument.\n", stderr);
	usage(argv);
	exit(0);
    }

    while ((result =
	    getopt_long(argc, argv, "3wtrao:epS0:1:2:4:LTSsv",
			long_options, NULL)) != -1) {
	switch (result) {
	case '3':		/* 3d output */
	    pr->is3d = 1;
	    break;
	case 'w':		/* converts only waypoint */
	    pr->parseWpt = 1;
	    break;
	case 't':		/* converts only trackpoint */
	    pr->parseTrk = 1;
	    break;
	case 'r':		/* converts only route */
	    pr->parseRte = 1;
	    fputs("Sorry, this option is not implemented yet.\n", stderr);
	    break;
	case 'a':		/* converts all */
	    pr->parseWpt = 1;
	    pr->parseTrk = 1;
	    pr->parseRte = 1;
	    break;
	case 'o':		/* sets basename of output file */
	    strcpy(pr->output, optarg);
	    break;
	case 'e':		/* make output by each edges */
	    pr->isSeparated = 1;
	    break;
	case 'p':		/* make output by each edges */
	    pr->isPoint = 1;
	    break;
	case 'S':		/* doesn't make speed column */
	    pr->needsSpeed = 0;
	    break;
	case '0':		/* sets length unit */
	    strcpy(pr->lengthUnit, optarg);
	    break;
	case '1':		/* sets length unit for calculating speed */
	    strcpy(pr->speedLengthUnit, optarg);
	    break;
	case '2':		/* sets time unit for calculating speed */
	    strcpy(pr->speedTimeUnit, optarg);
	    break;
	case '4':		/* sets ellipsoid for calculating length */
	    strcpy(pr->ellipsoid, optarg);
	    break;
	case 'L':		/* doesn't make length column */
	    pr->needsLength = 0;
	    break;
	case 'T':		/* doesn't make time column */
	    pr->needsTime = 0;
	    break;
	case 's':		/* shows source file stats */
	    pr->needsStats = 1;
	    break;
	case 'v':		/* verbose mode */
	    pr->verbose = 1;
	    break;
	case ':':
	    usage(argv);
	    exit(0);
	    break;
	default:
	    usage(argv);
	    exit(0);
	    break;
	}
    }
/*
 * gets a source file name 
 */
    for (; optind < argc; optind++)
	strcpy(pr->sourcefile, argv[optind]);
    setDefault(pr);
}


g2sprop
cleanProp(g2sprop pr)
{
/*
 * cleans pointers and etc of properties
 */
#ifdef DEBUG
    printf("(cleanProp)\n");
#endif
    pr.hDBF_trk = NULL;
    pr.hSHP_trk = NULL;
    pr.hDBF_wpt = NULL;
    pr.hSHP_wpt = NULL;
    pr.hDBF_rte = NULL;
    pr.hSHP_rte = NULL;
    pr.hDBF_trk_spt = NULL;
    pr.hSHP_trk_spt = NULL;
    pr.hDBF_rte_spt = NULL;
    pr.hSHP_rte_spt = NULL;
    pr.hDBF_trk_pnt = NULL;
    pr.hSHP_trk_pnt = NULL;
    pr.hDBF_rte_pnt = NULL;
    pr.hSHP_rte_pnt = NULL;
    pr.parseWpt = 0;
    pr.parseTrk = 0;
    pr.parseRte = 0;
    pr.is3d = 0;
    pr.isSeparated = 0;
    pr.isPoint = 0;
    pr.needsSpeed = 1;
    pr.needsLength = 1;
    pr.needsTime = 1;
    pr.needsStats = 0;
    pr.verbose = 0;
/*
 * pr->verbose = 1; 
 */
    pr.output[0] = '\0';
    pr.ellipsoid[0] = '\0';
    pr.lengthUnit[0] = '\0';
    pr.speedLengthUnit[0] = '\0';
    pr.speedTimeUnit[0] = '\0';
    pr.output[0] = '\0';
    pr.stats->trkcount = 0;
    pr.stats->trkpoints = 0;
    pr.stats->trklength = 0;
    pr.stats->rtecount = 0;
    pr.stats->rtepoints = 0;
    pr.stats->rtelength = 0;
    pr.stats->wptpoints = 0;
    return pr;
}

void
showStats(g2sprop * pr)
{
/*
 * shows source file short statistics
 */
#ifdef DEBUG
    printf("(showStats)\n");
#endif
    if (pr->needsStats) {
	if (pr->parseTrk) {
	    printf("Track Points:\n");
	    printf("\ttrack count:\t%i\n", pr->stats->trkcount);
	    printf("\tpoint count:\t%i\n", pr->stats->trkpoints);
	    printf("\ttotal length:\t%f\n", pr->stats->trklength);
	}
	if (pr->parseRte) {
	    printf("Routes:\n");
	    printf("\troute count:\t%i\n", pr->stats->rtecount);
	    printf("\tpoint count:\t%i\n", pr->stats->rtepoints);
	    printf("\ttotal length:\t%f\n", pr->stats->rtelength);
	}
	if (pr->parseWpt) {
	    printf("Waypoints:\n");
	    printf("\tpoint count:\t%i\n", pr->stats->wptpoints);
	}
    }
}
int
main(int argc, char **argv)
{
    g2sprop         pr;
    g2sstats        stats;
    pr.stats = &stats;
    pr = cleanProp(pr);
    setOptions(argc, argv, &pr);
    parseDoc(&pr);
    showStats(&pr);
    closeFiles(&pr);
    return (1);
}
